@@ -1,7 +1,10 @@
-__version__ = "0.3.3"
+__version__ = "0.4.0"
 from .bins import *
 from .perf import *
 from .stats import *
 from .utils import *
 from .props import *
 from .logit import *
+from .regression import *
+from .basics import *
+from .visualize import *
