@@ -4,10 +4,17 @@ import matplotlib.pyplot as plt
 import matplotlib.ticker as ticker
 from statsmodels.stats.outliers_influence import variance_inflation_factor
 import statsmodels.formula.api as smf
-from scipy.stats import norm
+from scipy import stats
+from scipy.special import expit
 from pyrsm.utils import ifelse
 
 
+def sig_stars(pval):
+    cutpoints = np.array([0.001, 0.01, 0.05, 0.1, 1])
+    symbols = np.array(["***", "**", "*", ".", " "])
+    return [symbols[p < cutpoints][0] for p in pval]
+
+
 def or_ci(fitted, alpha=0.05, intercept=False, dec=3):
     """
     Confidence interval for Odds ratios
@@ -17,8 +24,10 @@ def or_ci(fitted, alpha=0.05, intercept=False, dec=3):
     fitted : A fitted logistic regression model
     alpha : float
         Significance level
+    intercept : bool
+        Include intercept in output (True or False)
     dec : int
-        Number of decimal places
+        Number of decimal places to use in rounding
 
     Returns
     -------
@@ -31,9 +40,15 @@ def or_ci(fitted, alpha=0.05, intercept=False, dec=3):
     low, high = [100 * alpha / 2, 100 * (1 - (alpha / 2))]
     df[[f"{low}%", f"{high}%"]] = np.exp(fitted.conf_int(alpha=alpha))
 
-    if dec is not None:
+    if dec is None:
+        df["p.values"] = ifelse(fitted.pvalues < 0.001, "< .001", fitted.pvalues)
+    else:
         df = df.round(dec)
+        df["p.values"] = ifelse(
+            fitted.pvalues < 0.001, "< .001", fitted.pvalues.round(dec)
+        )
 
+    df["  "] = sig_stars(fitted.pvalues)
     df["OR%"] = [f"{OR}%" for OR in df["OR%"]]
     df = df.reset_index()
 
@@ -45,10 +60,10 @@ def or_ci(fitted, alpha=0.05, intercept=False, dec=3):
 
 def or_conf_int(fitted, alpha=0.05, intercept=False, dec=3):
     """ Shortcut to or_ci """
-    return or_ci(fitted, alpha=0.05, intercept=False, dec=3)
+    return or_ci(fitted, alpha=0.05, intercept=False, dec=dec)
 
 
-def or_plot(fitted, alpha=0.05, intercept=False, figsize=None):
+def or_plot(fitted, alpha=0.05, intercept=False, incl=None, excl=None, figsize=None):
     """
     Odds ratio plot
 
@@ -58,7 +73,11 @@ def or_plot(fitted, alpha=0.05, intercept=False, figsize=None):
     alpha : float
         Significance level
     intercept : bool
-        Include intercept in plot (True or False)
+        Include intercept in odds-ratio plot (True or False)
+    incl : str or list of strings
+        Variables to include in the odds-ratio plot. All will be included by default
+    excl : str or list of strings
+        Variables to exclude from the odds-ratio plot. None are excluded by default
 
     Returns
     -------
@@ -66,7 +85,23 @@ def or_plot(fitted, alpha=0.05, intercept=False, figsize=None):
         Plot of Odds ratios
     """
 
-    df = or_conf_int(fitted, alpha=alpha, intercept=intercept, dec=None).iloc[::-1]
+    df = or_ci(fitted, alpha=alpha, intercept=intercept, dec=None).iloc[::-1]
+
+    if incl is not None:
+        incl = ifelse(isinstance(incl, list), incl, [incl])
+        rx = "(" + "|".join([f"^\b{v}|^{v}\\[" for v in incl]) + ")"
+        incl = df["index"].str.match(fr"{rx}")
+        if intercept:
+            incl[0] = True
+        df = df[incl]
+
+    if excl is not None:
+        excl = ifelse(isinstance(excl, list), excl, [excl])
+        rx = "(" + "|".join([f"^\b{v}|^{v}\\[" for v in excl]) + ")"
+        excl = df["index"].str.match(fr"{rx}")
+        if intercept:
+            excl[0] = False
+        df = df[~excl]
 
     low, high = [100 * alpha / 2, 100 * (1 - (alpha / 2))]
     err = [df["OR"] - df[f"{low}%"], df[f"{high}%"] - df["OR"]]
@@ -177,14 +212,56 @@ def predict_ci(fitted, df, alpha=0.05):
     low, high = [alpha / 2, 1 - (alpha / 2)]
     Xb = np.dot(df, fitted.params)
     se = np.sqrt((df.dot(fitted.cov_params()) * df).sum(-1))
-    me = norm.ppf(high) * se
-    lb = np.exp(Xb - me)
-    ub = np.exp(Xb + me)
+    me = stats.norm.ppf(high) * se
 
     return pd.DataFrame(
         {
             "prediction": prediction,
-            f"{low*100}%": lb / (1 + lb),
-            f"{high*100}%": ub / (1 + ub),
+            f"{low*100}%": expit(Xb - me),
+            f"{high*100}%": expit(Xb + me),
         }
     )
+
+
+def model_fit(fitted, dec=3, prn=True):
+    """
+    Compute various model fit statistics for a fitted logistic regression model
+
+    Parameters
+    ----------
+    fitted : statmodels glm object
+        Logistic regression model fitted using statsmodels
+    dec : int
+        Number of decimal places to use in rounding
+    prn : bool
+        If True, print output, else return a Pandas dataframe with the results
+
+    Returns
+    -------
+        If prn is True, print output, else return a Pandas dataframe with the results
+    """
+
+    mfit = pd.DataFrame().assign(
+        pseudo_rsq_mcf=[1 - fitted.llf / fitted.llnull],
+        pseudo_rsq_mcf_adj=[1 - (fitted.llf - fitted.df_model) / fitted.llnull],
+        log_likelihood=fitted.llf,
+        BIC=[fitted.bic_llf],
+        AIC=[fitted.aic],
+        chisq=[fitted.pearson_chi2],
+        chisq_df=[fitted.df_model],
+        chisq_pval=[1 - stats.chi2.cdf(fitted.pearson_chi2, fitted.df_model)],
+        nobs=[fitted.nobs],
+    )
+
+    output = f"""
+Pseudo R-squared (McFadden): {mfit["pseudo_rsq_mcf"].values[0].round(dec)}
+Pseudo R-squared (McFadden adjusted): {mfit["pseudo_rsq_mcf_adj"].values[0].round(dec)}
+Log-likelihood: {mfit["log_likelihood"].values[0].round(dec)}, AIC: {mfit["AIC"].values[0].round(dec)}, BIC: {mfit["BIC"].values[0].round(dec)}
+Chi-squared: {mfit["chisq"].values[0].round(dec)} df({mfit["chisq_df"].values[0]}), p.value {np.where(mfit["chisq_pval"].values[0] < .001, "< 0.001", mfit["chisq_pval"].values[0].round(dec))} 
+Nr obs: {mfit["nobs"].values[0]:,}
+"""
+
+    if prn:
+        print(output)
+    else:
+        return mfit
